#include <stdio.h>
#include <sys/sysctl.h>
#include <errno.h>
#include <stdlib.h>
#include <limits.h>
#include "shadowmac.h"
extern int errno;

void usage();
int main(int argc, char **argv)
{
	int  ret;
	struct jc_shadowmac_data shadowmac_data;
	u_char addr[6];
	u_char shadow_addr[6];



	if (argc <= 1)
	{
		usage();
		exit(0);
	}
	
	//normal users can only print.
	if (strcmp(argv[1], "-p") != 0)
	{
		if (geteuid() != 0)
		{
			printf("Error. You must be root to do that.\n");
			exit(0);
		}
	}	

	
	if (strcmp(argv[1], "-l") == 0) /* set logging level */
	{
		if (argc != 3)
		{
			usage();
			exit(0);
		}
		int status = atoi(argv[2]);
		if (status == 0 && errno == EINVAL)
		{
			printf("Error. cant parse: %s\n", argv[2]);
			exit(0);
		}
		if (status < 0 || status > 2)
		{
			printf("Invalid logging status. 0-2 are legal\n");
			exit(0);
		}

		ret = set_shadowmac_log_status(status);
		if (!ret)
			printf("Error: Could not set status. \n");
		else
			printf("shadowmac log status set to %d\n", status);

		exit(0);
	}

	if (strcmp(argv[1], "-e") == 0) /* enable shadowmac in kernel */
	{
		if (argc != 2)
		{
			usage();
			exit(0);
		}
		
		ret = set_shadowmac_status(1);
		if (!ret)
			printf("Error: Could not set status. \n");
		else
			printf("shadowmac enabled\n");
		exit(0);
	}

	else if (strcmp(argv[1], "-d") == 0) /* disable shadowmac in kernel */
	{
		if (argc != 2)
		{
			usage();
			exit(0);
		}
		
		ret = set_shadowmac_status(0);
		if (!ret)
			printf("Error, could not set status\n");
		else
			printf("shadowmac disabled\n");
		exit(0);
	}

	else if(strcmp(argv[1], "-p") == 0) /* print table */
	{
		if (argc != 2)
		{
			usage();
			exit(0);
		}
		printf("Enabled: %d\n", get_shadowmac_status());
		printf("Logging: %d\n", get_shadowmac_log_status());
		
		get_shadow_table(&shadowmac_data);
		print_shadow_table(&shadowmac_data);
		exit(0);
	}
	else if (strcmp(argv[1], "-s") == 0) /* set table */
	{
		if (argc != 4)
		{
			usage();
			exit(0);	
		}
		
		ret = parse_address(argv[2], addr); //parse mac address
		if (!ret)
		{
			printf("Couldnt parse %s:\n", argv[2]);
			exit(0);
		}

		ret = parse_address(argv[3], shadow_addr); //parse mac address
		if (!ret)
		{
			printf("Couldnt parse %s:\n", argv[3]);
			exit(0);
		}

		//fill the table with new data
		memcpy(shadowmac_data.mac, &addr, sizeof(shadowmac_data.mac));
		memcpy(shadowmac_data.shadow_mac, &shadow_addr, sizeof(shadowmac_data.shadow_mac));
		// put the new table into the kernel
		ret = set_shadow_table(&shadowmac_data);
		if (!ret)
		{
			printf("Couldnt  set table in kernel.\n");
			exit(0);
		}

		// read it back
		ret = get_shadow_table(&shadowmac_data);
		if (!ret)
		{
			printf("Couldnt get table from kernel.\n");
			exit(0);
		}
		print_shadow_table(&shadowmac_data);
	
		exit(0);
	}
	else
	{
		usage();
		exit(0);
	}
	
}


void usage()
{
	printf(" jc-shadowmac a nologin.org production\n\n");
	printf("-e: enable shadowmac support in kernel\n");
	printf("-d: disable shadowmac support in kernel\n");
	printf("-p: print table\n");
	printf("-s: set shadowmac data\n");
	printf("-l (0,1,2): set logging level (none, some, alot)\n"); 
}

